# -*- coding: utf-8 -*-

# Copyright (c) 2016, Brandon Nielsen
# All rights reserved.
#
# This software may be modified and distributed under the terms
# of the BSD license.  See the LICENSE file for details.

from datetime import datetime
from aniso8601.duration import parse_duration
from aniso8601.time import parse_datetime
from aniso8601.date import parse_date

def parse_interval(isointervalstr, intervaldelimiter='/', datetimedelimiter='T', relative=False):
    #Given a string representing an ISO 8601 interval, return a
    #tuple of datetime.date or date.datetime objects representing the beginning
    #and end of the specified interval. Valid formats are:
    #
    #<start>/<end>
    #<start>/<duration>
    #<duration>/<end>
    #
    #The <start> and <end> values can represent dates, or datetimes,
    #not times.
    #
    #The format:
    #
    #<duration>
    #
    #Is expressly not supported as there is no way to provide the addtional
    #required context.

    interval_parts = _parse_interval_parts(isointervalstr, intervaldelimiter, datetimedelimiter, relative)

    return (interval_parts[0], interval_parts[1])

def parse_repeating_interval(isointervalstr, intervaldelimiter='/', datetimedelimiter='T', relative=False):
    #Given a string representing an ISO 8601 interval repating, return a
    #generator of datetime.date or date.datetime objects representing the
    #dates specified by the repeating interval. Valid formats are:
    #
    #Rnn/<interval>
    #R/<interval>

    if isointervalstr[0] != 'R':
        raise ValueError('ISO 8601 repeating interval must start with an R.')

    #Parse the number of iterations
    iterationpart, intervalpart = isointervalstr.split(intervaldelimiter, 1)

    if len(iterationpart) > 1:
        iterations = int(iterationpart[1:])
    else:
        iterations = None

    interval_parts = _parse_interval_parts(intervalpart, intervaldelimiter, datetimedelimiter, relative=relative)

    #Now, build and return the generator
    if iterations != None:
        return _date_generator(interval_parts[0], interval_parts[2], iterations)
    else:
        return _date_generator_unbounded(interval_parts[0], interval_parts[2])

def _parse_interval_parts(isointervalstr, intervaldelimiter='/', datetimedelimiter='T', relative=False):
    #Returns a tuple containing the start of the interval, the end of the interval, and the interval timedelta
    firstpart, secondpart = isointervalstr.split(intervaldelimiter)

    if firstpart[0] == 'P':
        #<duration>/<end>
        #Notice that these are not returned 'in order' (earlier to later), this
        #is to maintain consistency with parsing <start>/<end> durations, as
        #well as making repeating interval code cleaner. Users who desire
        #durations to be in order can use the 'sorted' operator.

        #We need to figure out if <end> is a date, or a datetime
        if secondpart.find(datetimedelimiter) != -1:
            #<end> is a datetime
            duration = parse_duration(firstpart, relative=relative)
            enddatetime = parse_datetime(secondpart, delimiter=datetimedelimiter)

            return (enddatetime, enddatetime - duration, -duration)
        else:
            #<end> must just be a date
            duration = parse_duration(firstpart, relative=relative)
            enddate = parse_date(secondpart)

            #See if we need to upconvert to datetime to preserve resolution
            if firstpart.find(datetimedelimiter) != -1:
                return (enddate, datetime.combine(enddate, datetime.min.time()) - duration, -duration)
            else:
                return (enddate, enddate - duration, -duration)
    elif secondpart[0] == 'P':
        #<start>/<duration>
        #We need to figure out if <start> is a date, or a datetime
        if firstpart.find(datetimedelimiter) != -1:
            #<start> is a datetime
            duration = parse_duration(secondpart, relative=relative)
            startdatetime = parse_datetime(firstpart, delimiter=datetimedelimiter)

            return (startdatetime, startdatetime + duration, duration)
        else:
            #<start> must just be a date
            duration = parse_duration(secondpart, relative=relative)
            startdate = parse_date(firstpart)

            #See if we need to upconvert to datetime to preserve resolution
            if secondpart.find(datetimedelimiter) != -1:
                return (startdate, datetime.combine(startdate, datetime.min.time()) + duration, duration)
            else:
                return (startdate, startdate + duration, duration)
    else:
        #<start>/<end>
        if firstpart.find(datetimedelimiter) != -1 and secondpart.find(datetimedelimiter) != -1:
            #Both parts are datetimes
            start_datetime = parse_datetime(firstpart, delimiter=datetimedelimiter)
            end_datetime = parse_datetime(secondpart, delimiter=datetimedelimiter)

            return (start_datetime, end_datetime, end_datetime - start_datetime)
        elif firstpart.find(datetimedelimiter) != -1 and secondpart.find(datetimedelimiter) == -1:
            #First part is a datetime, second part is a date
            start_datetime = parse_datetime(firstpart, delimiter=datetimedelimiter)
            end_date = parse_date(secondpart)

            return (start_datetime, end_date, datetime.combine(end_date, datetime.min.time()) - start_datetime)
        elif firstpart.find(datetimedelimiter) == -1 and secondpart.find(datetimedelimiter) != -1:
            #First part is a date, second part is a datetime
            start_date = parse_date(firstpart)
            end_datetime = parse_datetime(secondpart, delimiter=datetimedelimiter)

            return (start_date, end_datetime, end_datetime - datetime.combine(start_date, datetime.min.time()))
        else:
            #Both parts are dates
            start_date = parse_date(firstpart)
            end_date = parse_date(secondpart)

            return (start_date, end_date, end_date - start_date)

def _date_generator(startdate, timedelta, iterations):
    currentdate = startdate
    currentiteration = 0

    while currentiteration < iterations:
        yield currentdate

        #Update the values
        currentdate += timedelta
        currentiteration += 1

def _date_generator_unbounded(startdate, timedelta):
    currentdate = startdate

    while True:
        yield currentdate

        #Update the value
        currentdate += timedelta



# -*- coding: utf-8 -*-

# Copyright (c) 2016, Brandon Nielsen
# All rights reserved.
#
# This software may be modified and distributed under the terms
# of the BSD license.  See the LICENSE file for details.

import datetime

from aniso8601.date import parse_date
from aniso8601.time import parse_time
from aniso8601 import compat

def parse_duration(isodurationstr, relative=False):
    #Given a string representing an ISO 8601 duration, return a
    #datetime.timedelta (or dateutil.relativedelta.relativedelta
    #if relative=True) that matches the given duration. Valid formats are:
    #
    #PnYnMnDTnHnMnS (or any reduced precision equivalent)
    #P<date>T<time>

    if isodurationstr[0] != 'P':
        raise ValueError('ISO 8601 duration must start with a P.')

    #If Y, M, D, H, S, or W are in the string, assume it is a specified duration
    if _has_any_component(isodurationstr, ['Y', 'M', 'D', 'H', 'S', 'W']) is True:
        return _parse_duration_prescribed(isodurationstr, relative)
    else:
        return _parse_duration_combined(isodurationstr, relative)

def _parse_duration_prescribed(durationstr, relative):
    #durationstr can be of the form PnYnMnDTnHnMnS or PnW

    #Make sure the end character is valid
    #https://bitbucket.org/nielsenb/aniso8601/issues/9/durations-with-trailing-garbage-are-parsed
    if durationstr[-1] not in ['Y', 'M', 'D', 'H', 'S', 'W']:
        raise ValueError('ISO 8601 duration must end with a valid character.')

    #Make sure only the lowest order element has decimal precision
    if durationstr.count('.') > 1:
        raise ValueError('ISO 8601 allows only lowest order element to have a decimal fraction.')
    elif durationstr.count('.') == 1:
        #There should only ever be 1 letter after a decimal if there is more
        #then one, the string is invalid
        lettercount = 0;

        for character in durationstr.split('.')[1]:
            if character.isalpha() is True:
                lettercount += 1

            if lettercount > 1:
                raise ValueError('ISO 8601 duration must end with a single valid character.')

    #Do not allow W in combination with other designators
    #https://bitbucket.org/nielsenb/aniso8601/issues/2/week-designators-should-not-be-combinable
    if durationstr.find('W') != -1 and _has_any_component(durationstr, ['Y', 'M', 'D', 'H', 'S']) is True:
        raise ValueError('ISO 8601 week designators may not be combined with other time designators.')

    #Parse the elements of the duration
    if durationstr.find('T') == -1:
        years, months, weeks, days, hours, minutes, seconds = _parse_duration_prescribed_notime(durationstr)
    else:
        years, months, weeks, days, hours, minutes, seconds = _parse_duration_prescribed_time(durationstr)

    if relative is True:
        try:
            import dateutil.relativedelta

            if int(years) != years or int(months) != months:
                #https://github.com/dateutil/dateutil/issues/40
                raise ValueError('Fractional months and years are not defined for relative intervals.')

            return dateutil.relativedelta.relativedelta(years=int(years), months=int(months), weeks=weeks, days=days, hours=hours, minutes=minutes, seconds=seconds)
        except ImportError:
            raise RuntimeError('dateutil must be installed for relative duration support.')

    #Note that weeks can be handled without conversion to days
    totaldays = years * 365 + months * 30 + days

    return datetime.timedelta(weeks=weeks, days=totaldays, hours=hours, minutes=minutes, seconds=seconds)

def _parse_duration_prescribed_notime(durationstr):
    #durationstr can be of the form PnYnMnD or PnW

    #Make sure no time portion is included
    #https://bitbucket.org/nielsenb/aniso8601/issues/7/durations-with-time-components-before-t
    if _has_any_component(durationstr, ['H', 'S']):
        raise ValueError('ISO 8601 time components not allowed in duration without prescribed time.')

    if _component_order_correct(durationstr, ['P', 'Y', 'M', 'D', 'W']) is False:
        raise ValueError('ISO 8601 duration components must be in the correct order.')

    if durationstr.find('Y') != -1:
        years = _parse_duration_element(durationstr, 'Y')
    else:
        years = 0

    if durationstr.find('M') != -1:
        months = _parse_duration_element(durationstr, 'M')
    else:
        months = 0

    if durationstr.find('W') != -1:
        weeks = _parse_duration_element(durationstr, 'W')
    else:
        weeks = 0

    if durationstr.find('D') != -1:
        days = _parse_duration_element(durationstr, 'D')
    else:
        days = 0

    #No hours, minutes or seconds
    hours = 0
    minutes = 0
    seconds = 0

    return (years, months, weeks, days, hours, minutes, seconds)

def _parse_duration_prescribed_time(durationstr):
    #durationstr can be of the form PnYnMnDTnHnMnS

    firsthalf = durationstr[:durationstr.find('T')]
    secondhalf = durationstr[durationstr.find('T'):]

    #Make sure no time portion is included in the date half
    #https://bitbucket.org/nielsenb/aniso8601/issues/7/durations-with-time-components-before-t
    if _has_any_component(firsthalf, ['H', 'S']):
        raise ValueError('ISO 8601 time components not allowed in date portion of duration.')

    if _component_order_correct(firsthalf, ['P', 'Y', 'M', 'D', 'W']) is False:
        raise ValueError('ISO 8601 duration components must be in the correct order.')

    #Make sure no date component is included in the time half
    if _has_any_component(secondhalf, ['Y', 'D']):
        raise ValueError('ISO 8601 time components not allowed in date portion of duration.')

    if _component_order_correct(secondhalf, ['T', 'H', 'M', 'S']) is False:
        raise ValueError('ISO 8601 time components in duration must be in the correct order.')

    if firsthalf.find('Y') != -1:
        years = _parse_duration_element(firsthalf, 'Y')
    else:
        years = 0

    if firsthalf.find('M') != -1:
        months = _parse_duration_element(firsthalf, 'M')
    else:
        months = 0

    if firsthalf.find('D') != -1:
        days = _parse_duration_element(firsthalf, 'D')
    else:
        days = 0

    if secondhalf.find('H') != -1:
        hours = _parse_duration_element(secondhalf, 'H')
    else:
        hours = 0

    if secondhalf.find('M') != -1:
        minutes = _parse_duration_element(secondhalf, 'M')
    else:
        minutes = 0

    if secondhalf.find('S') != -1:
        seconds = _parse_duration_element(secondhalf, 'S')
    else:
        seconds = 0

    #Weeks can't be included
    weeks = 0

    return (years, months, weeks, days, hours, minutes, seconds)

def _parse_duration_combined(durationstr, relative):
    #Period of the form P<date>T<time>

    #Split the string in to its component parts
    datepart, timepart = durationstr[1:].split('T') #We skip the 'P'

    datevalue = parse_date(datepart)
    timevalue = parse_time(timepart)

    if relative is True:
        try:
            import dateutil.relativedelta

            return dateutil.relativedelta.relativedelta(years=datevalue.year, months=datevalue.month, days=datevalue.day, hours=timevalue.hour, minutes=timevalue.minute, seconds=timevalue.second, microseconds=timevalue.microsecond)
        except ImportError:
            raise RuntimeError('dateutil must be installed for relative duration support.')
    else:
        totaldays = datevalue.year * 365 + datevalue.month * 30 + datevalue.day

        return datetime.timedelta(days=totaldays, hours=timevalue.hour, minutes=timevalue.minute, seconds=timevalue.second, microseconds=timevalue.microsecond)

def _parse_duration_element(durationstr, elementstr):
    #Extracts the specified portion of a duration, for instance, given:
    #durationstr = 'T4H5M6.1234S'
    #elementstr = 'H'
    #
    #returns 4
    #
    #Note that the string must start with a character, so its assumed the
    #full duration string would be split at the 'T'

    durationstartindex = 0
    durationendindex = durationstr.find(elementstr)

    for characterindex in compat.range(durationendindex - 1, 0, -1):
        if durationstr[characterindex].isalpha() is True:
            durationstartindex = characterindex
            break

    durationstartindex += 1

    if ',' in durationstr:
        #Replace the comma with a 'full-stop'
        durationstr = durationstr.replace(',', '.')

    return float(durationstr[durationstartindex:durationendindex])

def _has_any_component(durationstr, components):
    #Given a duration string, and a list of components, returns True
    #if any of the listed components are present, False otherwise.
    #
    #For instance:
    #durationstr = 'P1Y'
    #components = ['Y', 'M']
    #
    #returns True
    #
    #durationstr = 'P1Y'
    #components = ['M', 'D']
    #
    #returns False

    for component in components:
        if durationstr.find(component) != -1:
            return True

    return False

def _component_order_correct(durationstr, componentorder):
    #Given a duration string, and a list of components, returns
    #True if the components are in the same order as the
    #component order list, False otherwise. Characters that
    #are present in the component order list but not in the
    #duration string are ignored.
    #
    #https://bitbucket.org/nielsenb/aniso8601/issues/8/durations-with-components-in-wrong-order
    #
    #durationstr = 'P1Y1M1D'
    #components = ['P', 'Y', 'M', 'D']
    #
    #returns True
    #
    #durationstr = 'P1Y1M'
    #components = ['P', 'Y', 'M', 'D']
    #
    #returns True
    #
    #durationstr = 'P1D1Y1M'
    #components = ['P', 'Y', 'M', 'D']
    #
    #returns False

    componentindex = 0

    for characterindex in compat.range(len(durationstr)):
        character = durationstr[characterindex]

        if character in componentorder:
            #This is a character we need to check the order of
            if character in componentorder[componentindex:]:
                componentindex = componentorder.index(character)
            else:
                #A character is out of order
                return False

    return True



# -*- coding: utf-8 -*-

# Copyright (c) 2016, Brandon Nielsen
# All rights reserved.
#
# This software may be modified and distributed under the terms
# of the BSD license.  See the LICENSE file for details.

import datetime
import unittest

from aniso8601.date import parse_date, _parse_year, _parse_calendar_day, _parse_calendar_month, _parse_week_day, _parse_week, _parse_ordinal_date, get_date_resolution
from aniso8601.resolution import DateResolution

class TestDateResolutionFunctions(unittest.TestCase):
    def test_get_date_resolution_year(self):
        self.assertEqual(get_date_resolution('2013'), DateResolution.Year)
        self.assertEqual(get_date_resolution('0001'), DateResolution.Year)
        self.assertEqual(get_date_resolution('19'), DateResolution.Year)

    def test_get_date_resolution_month(self):
        self.assertEqual(get_date_resolution('1981-04'), DateResolution.Month)

    def test_get_date_resolution_week(self):
        self.assertEqual(get_date_resolution('2004-W53'), DateResolution.Week)
        self.assertEqual(get_date_resolution('2009-W01'), DateResolution.Week)
        self.assertEqual(get_date_resolution('2004W53'), DateResolution.Week)

    def test_get_date_resolution_year_weekday(self):
        self.assertEqual(get_date_resolution('2004-W53-6'), DateResolution.Weekday)
        self.assertEqual(get_date_resolution('2004W536'), DateResolution.Weekday)

    def test_get_date_resolution_year_ordinal(self):
        self.assertEqual(get_date_resolution('1981-095'), DateResolution.Ordinal)
        self.assertEqual(get_date_resolution('1981095'), DateResolution.Ordinal)

class TestDateParserFunctions(unittest.TestCase):
    def test_parse_date(self):
        date = parse_date('2013')
        self.assertEqual(date, datetime.date(2013, 1, 1))

        date = parse_date('0001')
        self.assertEqual(date, datetime.date(1, 1, 1))

        date = parse_date('19')
        self.assertEqual(date, datetime.date(1900, 1, 1))

        date = parse_date('1981-04-05')
        self.assertEqual(date, datetime.date(1981, 4, 5))

        date = parse_date('19810405')
        self.assertEqual(date, datetime.date(1981, 4, 5))

        date = parse_date('1981-04')
        self.assertEqual(date, datetime.date(1981, 4, 1))

        date = parse_date('2004-W53')
        self.assertEqual(date, datetime.date(2004, 12, 27))
        self.assertEqual(date.weekday(), 0)

        date = parse_date('2009-W01')
        self.assertEqual(date, datetime.date(2008, 12, 29))
        self.assertEqual(date.weekday(), 0)

        date = parse_date('2004-W53-6')
        self.assertEqual(date, datetime.date(2005, 1, 1))
        self.assertEqual(date.weekday(), 5)

        date = parse_date('2004W53')
        self.assertEqual(date, datetime.date(2004, 12, 27))
        self.assertEqual(date.weekday(), 0)

        date = parse_date('2004W536')
        self.assertEqual(date, datetime.date(2005, 1, 1))
        self.assertEqual(date.weekday(), 5)

        date = parse_date('1981-095')
        self.assertEqual(date, datetime.date(1981, 4, 5))

        date = parse_date('1981095')
        self.assertEqual(date, datetime.date(1981, 4, 5))

    def test_parse_year(self):
        date = _parse_year('2013')
        self.assertEqual(date, datetime.date(2013, 1, 1))

        date = _parse_year('0001')
        self.assertEqual(date, datetime.date(1, 1, 1))

        date = _parse_year('19')
        self.assertEqual(date, datetime.date(1900, 1, 1))

    def test_parse_year_zero(self):
        #0 isn't a valid year
        with self.assertRaises(ValueError):
            _parse_year('0')

    def test_parse_calendar_day(self):
        date = _parse_calendar_day('1981-04-05')
        self.assertEqual(date, datetime.date(1981, 4, 5))

        date = _parse_calendar_day('19810405')
        self.assertEqual(date, datetime.date(1981, 4, 5))

    def test_parse_calendar_month(self):
        date = _parse_calendar_month('1981-04')
        self.assertEqual(date, datetime.date(1981, 4, 1))

    def test_parse_calendar_month_nohyphen(self):
        #Hyphen is required
        with self.assertRaises(ValueError):
            _parse_calendar_month('198104')

    def test_parse_week_day(self):
        date = _parse_week_day('2004-W53-6')
        self.assertEqual(date, datetime.date(2005, 1, 1))
        self.assertEqual(date.weekday(), 5)

        date = _parse_week_day('2009-W01-1')
        self.assertEqual(date, datetime.date(2008, 12, 29))
        self.assertEqual(date.weekday(), 0)

        date = _parse_week_day('2009-W53-7')
        self.assertEqual(date, datetime.date(2010, 1, 3))
        self.assertEqual(date.weekday(), 6)

        date = _parse_week_day('2010-W01-1')
        self.assertEqual(date, datetime.date(2010, 1, 4))
        self.assertEqual(date.weekday(), 0)

        date = _parse_week_day('2004W536')
        self.assertEqual(date, datetime.date(2005, 1, 1))
        self.assertEqual(date.weekday(), 5)

        date = _parse_week_day('2009W011')
        self.assertEqual(date, datetime.date(2008, 12, 29))
        self.assertEqual(date.weekday(), 0)

        date = _parse_week_day('2009W537')
        self.assertEqual(date, datetime.date(2010, 1, 3))
        self.assertEqual(date.weekday(), 6)

        date = _parse_week_day('2010W011')
        self.assertEqual(date, datetime.date(2010, 1, 4))
        self.assertEqual(date.weekday(), 0)

    def test_parse_week(self):
        date = _parse_week('2004-W53')
        self.assertEqual(date, datetime.date(2004, 12, 27))
        self.assertEqual(date.weekday(), 0)

        date = _parse_week('2009-W01')
        self.assertEqual(date, datetime.date(2008, 12, 29))
        self.assertEqual(date.weekday(), 0)

        date = _parse_week('2009-W53')
        self.assertEqual(date, datetime.date(2009, 12, 28))
        self.assertEqual(date.weekday(), 0)

        date = _parse_week('2010-W01')
        self.assertEqual(date, datetime.date(2010, 1, 4))
        self.assertEqual(date.weekday(), 0)

        date = _parse_week('2004W53')
        self.assertEqual(date, datetime.date(2004, 12, 27))
        self.assertEqual(date.weekday(), 0)

        date = _parse_week('2009W01')
        self.assertEqual(date, datetime.date(2008, 12, 29))
        self.assertEqual(date.weekday(), 0)

        date = _parse_week('2009W53')
        self.assertEqual(date, datetime.date(2009, 12, 28))
        self.assertEqual(date.weekday(), 0)

        date = _parse_week('2010W01')
        self.assertEqual(date, datetime.date(2010, 1, 4))
        self.assertEqual(date.weekday(), 0)

    def test_parse_ordinal_date(self):
        date = _parse_ordinal_date('1981-095')
        self.assertEqual(date, datetime.date(1981, 4, 5))

        date = _parse_ordinal_date('1981095')
        self.assertEqual(date, datetime.date(1981, 4, 5))

        date = _parse_ordinal_date('1981365')
        self.assertEqual(date, datetime.date(1981, 12, 31))

        date = _parse_ordinal_date('1980366')
        self.assertEqual(date, datetime.date(1980, 12, 31))

    def test_parse_ordinal_date_zero(self):
        #0 isn't a valid day of year
        with self.assertRaises(ValueError):
            _parse_ordinal_date('1981000')

    def test_parse_ordinal_date_nonleap(self):
        #Day 366 is only valid on a leap year
        with self.assertRaises(ValueError):
            _parse_ordinal_date('1981366')

    def test_parse_ordinal_date_overflow(self):
        #Day must me 365, or 366, not larger
        with self.assertRaises(ValueError):
            _parse_ordinal_date('1981367')



# -*- coding: utf-8 -*-

# Copyright (c) 2016, Brandon Nielsen
# All rights reserved.
#
# This software may be modified and distributed under the terms
# of the BSD license.  See the LICENSE file for details.

import datetime
import unittest
import dateutil

from aniso8601.duration import parse_duration, _parse_duration_prescribed, \
        _parse_duration_combined, _parse_duration_prescribed_notime, \
        _parse_duration_prescribed_time, _parse_duration_element, \
        _has_any_component, _component_order_correct

class TestDurationParserFunctions(unittest.TestCase):
    def test_parse_duration(self):
        resultduration = parse_duration('P1Y2M3DT4H54M6S')
        self.assertEqual(resultduration, datetime.timedelta(days=428, seconds=17646))

        resultduration = parse_duration('P1Y2M3DT4H54M6.5S')
        self.assertEqual(resultduration, datetime.timedelta(days=428, seconds=17646.5))

        resultduration = parse_duration('P1Y2M3DT4H54M6,5S')
        self.assertEqual(resultduration, datetime.timedelta(days=428, seconds=17646.5))

        resultduration = parse_duration('P1Y2M3D')
        self.assertEqual(resultduration, datetime.timedelta(days=428))

        resultduration = parse_duration('P1Y2M3.5D')
        self.assertEqual(resultduration, datetime.timedelta(days=428.5))

        resultduration = parse_duration('P1Y2M3,5D')
        self.assertEqual(resultduration, datetime.timedelta(days=428.5))

        resultduration = parse_duration('PT4H54M6.5S')
        self.assertEqual(resultduration, datetime.timedelta(hours=4, minutes=54, seconds=6.5))

        resultduration = parse_duration('PT4H54M6,5S')
        self.assertEqual(resultduration, datetime.timedelta(hours=4, minutes=54, seconds=6.5))

        resultduration = parse_duration('P1Y')
        self.assertEqual(resultduration, datetime.timedelta(days=365))

        resultduration = parse_duration('P1.5Y')
        self.assertEqual(resultduration, datetime.timedelta(days=547.5))

        resultduration = parse_duration('P1,5Y')
        self.assertEqual(resultduration, datetime.timedelta(days=547.5))

        resultduration = parse_duration('P1M')
        self.assertEqual(resultduration, datetime.timedelta(days=30))

        resultduration = parse_duration('P1.5M')
        self.assertEqual(resultduration, datetime.timedelta(days=45))

        resultduration = parse_duration('P1,5M')
        self.assertEqual(resultduration, datetime.timedelta(days=45))

        resultduration = parse_duration('P1W')
        self.assertEqual(resultduration, datetime.timedelta(days=7))

        resultduration = parse_duration('P1.5W')
        self.assertEqual(resultduration, datetime.timedelta(days=10.5))

        resultduration = parse_duration('P1,5W')
        self.assertEqual(resultduration, datetime.timedelta(days=10.5))

        resultduration = parse_duration('P1D')
        self.assertEqual(resultduration, datetime.timedelta(days=1))

        resultduration = parse_duration('P1.5D')
        self.assertEqual(resultduration, datetime.timedelta(days=1.5))

        resultduration = parse_duration('P1,5D')
        self.assertEqual(resultduration, datetime.timedelta(days=1.5))

        resultduration = parse_duration('P0003-06-04T12:30:05')
        self.assertEqual(resultduration, datetime.timedelta(days=1279, hours=12, minutes=30, seconds=5))

        resultduration = parse_duration('P0003-06-04T12:30:05.5')
        self.assertEqual(resultduration, datetime.timedelta(days=1279, hours=12, minutes=30, seconds=5.5))

        #Verify overflows
        self.assertEqual(parse_duration('PT36H'), parse_duration('P1DT12H'))

    def test_parse_duration_nop(self):
        with self.assertRaises(ValueError):
            #Duration must start with a P
            parse_duration('1Y2M3DT4H54M6S')

    def test_parse_duration_weekcombination(self):
        with self.assertRaises(ValueError):
            #Week designator cannot be combined with other time designators
            #https://bitbucket.org/nielsenb/aniso8601/issues/2/week-designators-should-not-be-combinable
            parse_duration('P1Y2W')

    def test_parse_duration_outoforder(self):
        #Ensure durations are required to be in the correct order
        #https://bitbucket.org/nielsenb/aniso8601/issues/7/durations-with-time-components-before-t
        #https://bitbucket.org/nielsenb/aniso8601/issues/8/durations-with-components-in-wrong-order
        with self.assertRaises(ValueError):
            parse_duration('P1S')

        with self.assertRaises(ValueError):
            parse_duration('P1D1S')

        with self.assertRaises(ValueError):
            parse_duration('P1H1M')

        with self.assertRaises(ValueError):
            parse_duration('1Y2M3D1SPT1M')

        with self.assertRaises(ValueError):
            parse_duration('P1Y2M3D2MT1S')

        with self.assertRaises(ValueError):
            parse_duration('P2M3D1ST1Y1M')

        with self.assertRaises(ValueError):
            parse_duration('P1Y2M2MT3D1S')

        with self.assertRaises(ValueError):
            parse_duration('P1D1Y1M')

        with self.assertRaises(ValueError):
            parse_duration('PT1S1H')

    def test_parse_duration_suffixgarbage(self):
        #Don't allow garbage after the duration
        #https://bitbucket.org/nielsenb/aniso8601/issues/9/durations-with-trailing-garbage-are-parsed
        with self.assertRaises(ValueError):
            parse_duration('P1Dasdfasdf')

        with self.assertRaises(ValueError):
            parse_duration('P0003-06-04T12:30:05.5asdfasdf')

    def test_parse_duration_prescribed(self):
        resultduration = _parse_duration_prescribed('P1Y2M3DT4H54M6S', False)
        self.assertEqual(resultduration, datetime.timedelta(days=428, hours=4, minutes=54, seconds=6))

        resultduration = _parse_duration_prescribed('P1Y2M3DT4H54M6.5S', False)
        self.assertEqual(resultduration, datetime.timedelta(days=428, hours=4, minutes=54, seconds=6.5))

        resultduration = _parse_duration_prescribed('P1Y2M3DT4H54M6,5S', False)
        self.assertEqual(resultduration, datetime.timedelta(days=428, hours=4, minutes=54, seconds=6.5))

        resultduration = _parse_duration_prescribed('PT4H54M6.5S', False)
        self.assertEqual(resultduration, datetime.timedelta(hours=4, minutes=54, seconds=6.5))

        resultduration = _parse_duration_prescribed('PT4H54M6,5S', False)
        self.assertEqual(resultduration, datetime.timedelta(hours=4, minutes=54, seconds=6.5))

        resultduration = _parse_duration_prescribed('P1Y2M3D', False)
        self.assertEqual(resultduration, datetime.timedelta(days=428))

        resultduration = _parse_duration_prescribed('P1Y2M3.5D', False)
        self.assertEqual(resultduration, datetime.timedelta(days=428.5))

        resultduration = _parse_duration_prescribed('P1Y2M3,5D', False)
        self.assertEqual(resultduration, datetime.timedelta(days=428.5))

        resultduration = _parse_duration_prescribed('P1Y', False)
        self.assertEqual(resultduration, datetime.timedelta(days=365))

        resultduration = _parse_duration_prescribed('P1.5Y', False)
        self.assertEqual(resultduration, datetime.timedelta(days=547.5))

        resultduration = _parse_duration_prescribed('P1,5Y', False)
        self.assertEqual(resultduration, datetime.timedelta(days=547.5))

        resultduration = _parse_duration_prescribed('P1M', False)
        self.assertEqual(resultduration, datetime.timedelta(days=30))

        resultduration = _parse_duration_prescribed('P1.5M', False)
        self.assertEqual(resultduration, datetime.timedelta(days=45))

        resultduration = _parse_duration_prescribed('P1,5M', False)
        self.assertEqual(resultduration, datetime.timedelta(days=45))

        resultduration = _parse_duration_prescribed('P1W', False)
        self.assertEqual(resultduration, datetime.timedelta(days=7))

        resultduration = _parse_duration_prescribed('P1.5W', False)
        self.assertEqual(resultduration, datetime.timedelta(days=10.5))

        resultduration = _parse_duration_prescribed('P1,5W', False)
        self.assertEqual(resultduration, datetime.timedelta(days=10.5))

        resultduration = _parse_duration_prescribed('P1D', False)
        self.assertEqual(resultduration, datetime.timedelta(days=1))

        resultduration = _parse_duration_prescribed('P1.5D', False)
        self.assertEqual(resultduration, datetime.timedelta(days=1.5))

        resultduration = _parse_duration_prescribed('P1,5D', False)
        self.assertEqual(resultduration, datetime.timedelta(days=1.5))

        #Verify overflows
        self.assertEqual(parse_duration('PT36H'), parse_duration('P1DT12H'))

    def test_parse_duration_prescribed_multiplefractions(self):
        with self.assertRaises(ValueError):
            #Multiple fractions are not allowed
            _parse_duration_prescribed('P1Y2M3DT4H5.1234M6.1234S', False)

    def test_parse_duration_prescribed_middlefraction(self):
        with self.assertRaises(ValueError):
            #Fraction only allowed on final component
            _parse_duration_prescribed('P1Y2M3DT4H5.1234M6S', False)

    def test_parse_duration_prescribed_suffixgarbage(self):
        #Don't allow garbage after the duration
        #https://bitbucket.org/nielsenb/aniso8601/issues/9/durations-with-trailing-garbage-are-parsed
        with self.assertRaises(ValueError):
            _parse_duration_prescribed('P1Dasdfasdf', False)

    def test_parse_duration_prescribed_notime(self):
        resultduration = _parse_duration_prescribed_notime('P1Y2M3D')
        self.assertEqual(resultduration, (1, 2, 0, 3, 0, 0, 0))

        resultduration = _parse_duration_prescribed_notime('P1Y2M3.5D')
        self.assertEqual(resultduration, (1, 2, 0, 3.5, 0, 0, 0))

        resultduration = _parse_duration_prescribed_notime('P1Y2M3,5D')
        self.assertEqual(resultduration, (1, 2, 0, 3.5, 0, 0, 0))

        resultduration = _parse_duration_prescribed_notime('P1Y')
        self.assertEqual(resultduration, (1, 0, 0, 0, 0, 0, 0))

        resultduration = _parse_duration_prescribed_notime('P1.5Y')
        self.assertEqual(resultduration, (1.5, 0, 0, 0, 0, 0, 0))

        resultduration = _parse_duration_prescribed_notime('P1,5Y')
        self.assertEqual(resultduration, (1.5, 0, 0, 0, 0, 0, 0))

        resultduration = _parse_duration_prescribed_notime('P1M')
        self.assertEqual(resultduration, (0, 1, 0, 0, 0, 0, 0))

        resultduration = _parse_duration_prescribed_notime('P1.5M')
        self.assertEqual(resultduration, (0, 1.5, 0, 0, 0, 0, 0))

        resultduration = _parse_duration_prescribed_notime('P1,5M')
        self.assertEqual(resultduration, (0, 1.5, 0, 0, 0, 0, 0))

        resultduration = _parse_duration_prescribed_notime('P1W')
        self.assertEqual(resultduration, (0, 0, 1, 0, 0, 0, 0))

        resultduration = _parse_duration_prescribed_notime('P1.5W')
        self.assertEqual(resultduration, (0, 0, 1.5, 0, 0, 0, 0))

        resultduration = _parse_duration_prescribed_notime('P1,5W')
        self.assertEqual(resultduration, (0, 0, 1.5, 0, 0, 0, 0))

        resultduration = _parse_duration_prescribed_notime('P1D')
        self.assertEqual(resultduration, (0, 0, 0, 1, 0, 0, 0))

        resultduration = _parse_duration_prescribed_notime('P1.5D')
        self.assertEqual(resultduration, (0, 0, 0, 1.5, 0, 0, 0))

        resultduration = _parse_duration_prescribed_notime('P1,5D')
        self.assertEqual(resultduration, (0, 0, 0, 1.5, 0, 0, 0))

    def test_parse_duration_prescribed_notime_timepart(self):
        #Ensure no time part is allowed
        with self.assertRaises(ValueError):
            _parse_duration_prescribed_notime('P1S')

        with self.assertRaises(ValueError):
            _parse_duration_prescribed_notime('P1D1S')

        with self.assertRaises(ValueError):
            _parse_duration_prescribed_notime('P1H1M')

        with self.assertRaises(ValueError):
            _parse_duration_prescribed_notime('P1Y2M3D4H')

        with self.assertRaises(ValueError):
            _parse_duration_prescribed_notime('P1Y2M3D4H5S')

    def test_parse_duration_prescribed_notime_outoforder(self):
        #Ensure durations are required to be in the correct order
        #https://bitbucket.org/nielsenb/aniso8601/issues/8/durations-with-components-in-wrong-order
        with self.assertRaises(ValueError):
            _parse_duration_prescribed_notime('P1H1M')

        with self.assertRaises(ValueError):
            _parse_duration_prescribed_notime('P1D1Y1M')

    def test_parse_duration_prescribed_time(self):
        resultduration = _parse_duration_prescribed_time('P1Y2M3DT4H54M6S')
        self.assertEqual(resultduration, (1, 2, 0, 3, 4, 54, 6))

        resultduration = _parse_duration_prescribed_time('P1Y2M3DT4H54M6.5S')
        self.assertEqual(resultduration, (1, 2, 0, 3, 4, 54, 6.5))

        resultduration = _parse_duration_prescribed_time('P1Y2M3DT4H54M6,5S')
        self.assertEqual(resultduration, (1, 2, 0, 3, 4, 54, 6.5))

        resultduration = _parse_duration_prescribed_time('PT4H54M6.5S')
        self.assertEqual(resultduration, (0, 0, 0, 0, 4, 54, 6.5))

        resultduration = _parse_duration_prescribed_time('PT4H54M6,5S')
        self.assertEqual(resultduration, (0, 0, 0, 0, 4, 54, 6.5))

    def test_parse_duration_prescribed_time_timeindate(self):
        #Don't allow time components in date half
        with self.assertRaises(ValueError):
            _parse_duration_prescribed_time('P1Y2M3D4HT54M6S')

        with self.assertRaises(ValueError):
            _parse_duration_prescribed_time('P1Y2M3D6ST4H54M')

    def test_parse_duration_prescribed_time_dateintime(self):
        #Don't allow date components in time half
        with self.assertRaises(ValueError):
            _parse_duration_prescribed_time('P2M3DT1Y4H54M6S')

        with self.assertRaises(ValueError):
            _parse_duration_prescribed_time('P1Y2MT3D4H54M6S')

    def test_parse_duration_prescribed_time_outoforder(self):
        #Ensure durations are required to be in the correct order
        #https://bitbucket.org/nielsenb/aniso8601/issues/7/durations-with-time-components-before-t
        with self.assertRaises(ValueError):
            _parse_duration_prescribed_time('1Y2M3D1SPT1M')

        with self.assertRaises(ValueError):
            _parse_duration_prescribed_time('P1Y2M3D2MT1S')

        with self.assertRaises(ValueError):
            _parse_duration_prescribed_time('P2M3D1ST1Y1M')

        with self.assertRaises(ValueError):
            _parse_duration_prescribed_time('P1Y2M2MT3D1S')

    def test_parse_duration_combined(self):
        resultduration = _parse_duration_combined('P0003-06-04T12:30:05', False)
        self.assertEqual(resultduration, datetime.timedelta(days=1279, hours=12, minutes=30, seconds=5))

        resultduration = _parse_duration_combined('P0003-06-04T12:30:05.5', False)
        self.assertEqual(resultduration, datetime.timedelta(days=1279, hours=12, minutes=30, seconds=5.5))

    def test_parse_duration_combined_suffixgarbage(self):
        #Don't allow garbage after the duration
        #https://bitbucket.org/nielsenb/aniso8601/issues/9/durations-with-trailing-garbage-are-parsed
        with self.assertRaises(ValueError):
            parse_duration('P0003-06-04T12:30:05.5asdfasdf')

    def test_parse_duration_element(self):
        self.assertEqual(_parse_duration_element('P1Y2M3D', 'Y'), 1)
        self.assertEqual(_parse_duration_element('P1Y2M3D', 'M'), 2)
        self.assertEqual(_parse_duration_element('P1Y2M3D', 'D'), 3)
        self.assertEqual(_parse_duration_element('T4H5M6.1234S', 'H'), 4)
        self.assertEqual(_parse_duration_element('T4H5M6.1234S', 'M'), 5)
        self.assertEqual(_parse_duration_element('T4H5M6.1234S', 'S'), 6.1234)
        self.assertEqual(_parse_duration_element('PT4H54M6,5S', 'H'), 4)
        self.assertEqual(_parse_duration_element('PT4H54M6,5S', 'M'), 54)
        self.assertEqual(_parse_duration_element('PT4H54M6,5S', 'S'), 6.5)

    def test_has_any_component(self):
        self.assertTrue(_has_any_component('P1Y', ['Y', 'M']))
        self.assertFalse(_has_any_component('P1Y', ['M', 'D']))

    def test_component_order_correct(self):
        self.assertTrue(_component_order_correct('P1Y1M1D', ['P', 'Y', 'M', 'D']))
        self.assertTrue(_component_order_correct('P1Y1M', ['P', 'Y', 'M', 'D']))
        self.assertFalse(_component_order_correct('P1D1Y1M', ['P', 'Y', 'M', 'D']))
        self.assertFalse(_component_order_correct('PT1S1H', ['T', 'H', 'M', 'S']))

class TestRelativeDurationParserFunctions(unittest.TestCase):
    def test_parse_duration_relative(self):
        resultduration = parse_duration('P1Y2M3DT4H54M6.5S', relative=True)
        self.assertEqual(resultduration, dateutil.relativedelta.relativedelta(years=1, months=2, days=3, hours=4, minutes=54, seconds=6.5))

        resultduration = parse_duration('P0003-06-04T12:30:05.5', relative=True)
        self.assertEqual(resultduration, dateutil.relativedelta.relativedelta(years=3, months=6, days=4, hours=12, minutes=30, seconds=5, microseconds=500000))

    def test_parse_duration_prescribed_relative(self):
        resultduration = _parse_duration_prescribed('P1Y', True)
        self.assertEqual(resultduration, dateutil.relativedelta.relativedelta(years=1))

        resultduration = _parse_duration_prescribed('P1M', True)
        self.assertEqual(resultduration, dateutil.relativedelta.relativedelta(months=1))

        #Add the relative ‘days’ argument to the absolute day. Notice that the ‘weeks’ argument is multiplied by 7 and added to ‘days’.
        #http://dateutil.readthedocs.org/en/latest/relativedelta.html
        resultduration = _parse_duration_prescribed('P1W', True)
        self.assertEqual(resultduration, dateutil.relativedelta.relativedelta(days=7))

        resultduration = _parse_duration_prescribed('P1.5W', True)
        self.assertEqual(resultduration, dateutil.relativedelta.relativedelta(days=10.5))

    def test_parse_duration_prescribed_relative_multiplefractions(self):
        with self.assertRaises(ValueError):
            #Multiple fractions are not allowed
            _parse_duration_prescribed('P1Y2M3DT4H5.1234M6.1234S', True)

    def test_parse_duration_prescribed_relative_middlefraction(self):
        with self.assertRaises(ValueError):
            #Fraction only allowed on final component
            _parse_duration_prescribed('P1Y2M3DT4H5.1234M6S', True)

    def test_parse_duration_prescribed_relative_suffixgarbage(self):
        #Don't allow garbage after the duration
        #https://bitbucket.org/nielsenb/aniso8601/issues/9/durations-with-trailing-garbage-are-parsed
        with self.assertRaises(ValueError):
            _parse_duration_prescribed('P1Dasdfasdf', True)

    def test_parse_duration_prescribed_relative_fractionalyear(self):
        #Fractional months and years are not defined
        #https://github.com/dateutil/dateutil/issues/40
        with self.assertRaises(ValueError):
            _parse_duration_prescribed('P1.5Y', True)

    def test_parse_duration_prescribed_relative_fractionalmonth(self):
        #Fractional months and years are not defined
        #https://github.com/dateutil/dateutil/issues/40
        with self.assertRaises(ValueError):
            _parse_duration_prescribed('P1.5M', True)

    def test_parse_duration_prescribed_relative_nodateutil(self):
        import sys
        import dateutil

        dateutil_import = dateutil

        sys.modules['dateutil'] = None

        with self.assertRaises(RuntimeError):
            _parse_duration_prescribed('P1Y', True)

        #Reinstall dateutil
        sys.modules['dateutil'] = dateutil

    def test_parse_duration_prescribed_relative_outoforder(self):
        #Ensure durations are required to be in the correct order
        #https://bitbucket.org/nielsenb/aniso8601/issues/7/durations-with-time-components-before-t
        #https://bitbucket.org/nielsenb/aniso8601/issues/8/durations-with-components-in-wrong-order
        with self.assertRaises(ValueError):
            _parse_duration_prescribed('P1S', True)

        with self.assertRaises(ValueError):
            _parse_duration_prescribed('P1D1S', True)

        with self.assertRaises(ValueError):
            _parse_duration_prescribed('P1H1M', True)

        with self.assertRaises(ValueError):
            _parse_duration_prescribed('1Y2M3D1SPT1M', True)

        with self.assertRaises(ValueError):
            _parse_duration_prescribed('P1Y2M3D2MT1S', True)

        with self.assertRaises(ValueError):
            _parse_duration_prescribed('P2M3D1ST1Y1M', True)

        with self.assertRaises(ValueError):
            _parse_duration_prescribed('P1Y2M2MT3D1S', True)

        with self.assertRaises(ValueError):
            _parse_duration_prescribed('P1D1Y1M', True)

        with self.assertRaises(ValueError):
            _parse_duration_prescribed('PT1S1H', True)

    def test_parse_duration_combined_relative(self):
        resultduration = _parse_duration_combined('P0003-06-04T12:30:05', True)
        self.assertEqual(resultduration, dateutil.relativedelta.relativedelta(years=3, months=6, days=4, hours=12, minutes=30, seconds=5))

        resultduration = _parse_duration_combined('P0003-06-04T12:30:05.5', True)
        self.assertEqual(resultduration, dateutil.relativedelta.relativedelta(years=3, months=6, days=4, hours=12, minutes=30, seconds=5, microseconds=500000))

    def test_parse_duration_combined_relative_suffixgarbage(self):
        #Don't allow garbage after the duration
        #https://bitbucket.org/nielsenb/aniso8601/issues/9/durations-with-trailing-garbage-are-parsed
        with self.assertRaises(ValueError):
            _parse_duration_combined('P0003-06-04T12:30:05.5asdfasdf', True)

    def test_parse_duration_combined_relative_nodateutil(self):
        import sys
        import dateutil

        dateutil_import = dateutil

        sys.modules['dateutil'] = None

        with self.assertRaises(RuntimeError):
            _parse_duration_combined('P0003-06-04T12:30:05', True)

        #Reinstall dateutil
        sys.modules['dateutil'] = dateutil



# -*- coding: utf-8 -*-

# Copyright (c) 2016, Brandon Nielsen
# All rights reserved.
#
# This software may be modified and distributed under the terms
# of the BSD license.  See the LICENSE file for details.

import unittest
import datetime
import dateutil.relativedelta

from aniso8601 import compat
from aniso8601.interval import parse_interval, parse_repeating_interval, _parse_interval_parts

class TestIntervalParserFunctions(unittest.TestCase):
    def test_parse_interval(self):
        resultinterval = parse_interval('P1M/1981-04-05T01:01:00')
        self.assertEqual(resultinterval[0], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))
        self.assertEqual(resultinterval[1], datetime.datetime(year=1981, month=3, day=6, hour=1, minute=1))

        resultinterval = parse_interval('P1M/1981-04-05')
        self.assertEqual(resultinterval[0], datetime.date(year=1981, month=4, day=5))
        self.assertEqual(resultinterval[1], datetime.date(year=1981, month=3, day=6))

        resultinterval = parse_interval('PT1H/2014-11-12')
        self.assertEqual(resultinterval[0], datetime.date(year=2014, month=11, day=12))
        self.assertEqual(resultinterval[1], datetime.datetime(year=2014, month=11, day=11, hour=23))

        resultinterval = parse_interval('PT4H54M6.5S/2014-11-12')
        self.assertEqual(resultinterval[0], datetime.date(year=2014, month=11, day=12))
        self.assertEqual(resultinterval[1], datetime.datetime(year=2014, month=11, day=11, hour=19, minute=5, second=53, microsecond=500000))

        resultinterval = parse_interval('1981-04-05T01:01:00/P1M1DT1M')
        self.assertEqual(resultinterval[0], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))
        self.assertEqual(resultinterval[1], datetime.datetime(year=1981, month=5, day=6, hour=1, minute=2))

        resultinterval = parse_interval('1981-04-05/P1M1D')
        self.assertEqual(resultinterval[0], datetime.date(year=1981, month=4, day=5))
        self.assertEqual(resultinterval[1], datetime.date(year=1981, month=5, day=6))

        resultinterval = parse_interval('2014-11-12/PT1H')
        self.assertEqual(resultinterval[0], datetime.date(year=2014, month=11, day=12))
        self.assertEqual(resultinterval[1], datetime.datetime(year=2014, month=11, day=12, hour=1, minute=0))

        resultinterval = parse_interval('2014-11-12/PT4H54M6.5S')
        self.assertEqual(resultinterval[0], datetime.date(year=2014, month=11, day=12))
        self.assertEqual(resultinterval[1], datetime.datetime(year=2014, month=11, day=12, hour=4, minute=54, second=6, microsecond=500000))

        resultinterval = parse_interval('1980-03-05T01:01:00/1981-04-05T01:01:00')
        self.assertEqual(resultinterval[0], datetime.datetime(year=1980, month=3, day=5, hour=1, minute=1))
        self.assertEqual(resultinterval[1], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))

        resultinterval = parse_interval('1980-03-05T01:01:00/1981-04-05')
        self.assertEqual(resultinterval[0], datetime.datetime(year=1980, month=3, day=5, hour=1, minute=1))
        self.assertEqual(resultinterval[1], datetime.date(year=1981, month=4, day=5))

        resultinterval = parse_interval('1980-03-05/1981-04-05T01:01:00')
        self.assertEqual(resultinterval[0], datetime.date(year=1980, month=3, day=5))
        self.assertEqual(resultinterval[1], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))

        resultinterval = parse_interval('1980-03-05/1981-04-05')
        self.assertEqual(resultinterval[0], datetime.date(year=1980, month=3, day=5))
        self.assertEqual(resultinterval[1], datetime.date(year=1981, month=4, day=5))

        resultinterval = parse_interval('1981-04-05/1980-03-05')
        self.assertEqual(resultinterval[0], datetime.date(year=1981, month=4, day=5))
        self.assertEqual(resultinterval[1], datetime.date(year=1980, month=3, day=5))

        resultinterval = parse_interval('1980-03-05T01:01:00--1981-04-05T01:01:00', intervaldelimiter='--')
        self.assertEqual(resultinterval[0], datetime.datetime(year=1980, month=3, day=5, hour=1, minute=1))
        self.assertEqual(resultinterval[1], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))

        resultinterval = parse_interval('1980-03-05 01:01:00/1981-04-05 01:01:00', datetimedelimiter=' ')
        self.assertEqual(resultinterval[0], datetime.datetime(year=1980, month=3, day=5, hour=1, minute=1))
        self.assertEqual(resultinterval[1], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))

    def test_parse_interval_suffixgarbage(self):
        #Don't allow garbage after the duration
        #https://bitbucket.org/nielsenb/aniso8601/issues/9/durations-with-trailing-garbage-are-parsed
        with self.assertRaises(ValueError):
            parse_interval('2001/P1Dasdf')

        with self.assertRaises(ValueError):
            parse_interval('P1Dasdf/2001')

        with self.assertRaises(ValueError):
            parse_interval('2001/P0003-06-04T12:30:05.5asdfasdf')

        with self.assertRaises(ValueError):
            parse_interval('P0003-06-04T12:30:05.5asdfasdf/2001')

class TestRelativeIntervalParserFunctions(unittest.TestCase):
    def test_parse_interval_relative(self):
        resultinterval = parse_interval('P1M/1981-04-05T01:01:00', relative=True)
        self.assertEqual(resultinterval[0], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))
        self.assertEqual(resultinterval[1], datetime.datetime(year=1981, month=3, day=5, hour=1, minute=1))

        resultinterval = parse_interval('P1M/1981-04-05', relative=True)
        self.assertEqual(resultinterval[0], datetime.date(year=1981, month=4, day=5))
        self.assertEqual(resultinterval[1], datetime.date(year=1981, month=3, day=5))

        resultinterval = parse_interval('PT1H/2014-11-12', relative=True)
        self.assertEqual(resultinterval[0], datetime.date(year=2014, month=11, day=12))
        self.assertEqual(resultinterval[1], datetime.datetime(year=2014, month=11, day=11, hour=23))

        resultinterval = parse_interval('PT4H54M6.5S/2014-11-12', relative=True)
        self.assertEqual(resultinterval[0], datetime.date(year=2014, month=11, day=12))
        self.assertEqual(resultinterval[1], datetime.datetime(year=2014, month=11, day=11, hour=19, minute=5, second=53, microsecond=500000))

        resultinterval = parse_interval('1981-04-05T01:01:00/P1M1DT1M', relative=True)
        self.assertEqual(resultinterval[0], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))
        self.assertEqual(resultinterval[1], datetime.datetime(year=1981, month=5, day=6, hour=1, minute=2))

        resultinterval = parse_interval('1981-04-05/P1M1D', relative=True)
        self.assertEqual(resultinterval[0], datetime.date(year=1981, month=4, day=5))
        self.assertEqual(resultinterval[1], datetime.date(year=1981, month=5, day=6))

        resultinterval = parse_interval('2014-11-12/PT1H', relative=True)
        self.assertEqual(resultinterval[0], datetime.date(year=2014, month=11, day=12))
        self.assertEqual(resultinterval[1], datetime.datetime(year=2014, month=11, day=12, hour=1, minute=0))

        resultinterval = parse_interval('2014-11-12/PT4H54M6.5S', relative=True)
        self.assertEqual(resultinterval[0], datetime.date(year=2014, month=11, day=12))
        self.assertEqual(resultinterval[1], datetime.datetime(year=2014, month=11, day=12, hour=4, minute=54, second=6, microsecond=500000))

        #Some relativedelta examples
        #http://dateutil.readthedocs.org/en/latest/examples.html#relativedelta-examples
        resultinterval = parse_interval('2003-01-27/P1M', relative=True)
        self.assertEqual(resultinterval[0], datetime.date(year=2003, month=1, day=27))
        self.assertEqual(resultinterval[1], datetime.date(year=2003, month=2, day=27))

        resultinterval = parse_interval('2003-01-31/P1M', relative=True)
        self.assertEqual(resultinterval[0], datetime.date(year=2003, month=1, day=31))
        self.assertEqual(resultinterval[1], datetime.date(year=2003, month=2, day=28))

        resultinterval = parse_interval('2003-01-31/P2M', relative=True)
        self.assertEqual(resultinterval[0], datetime.date(year=2003, month=1, day=31))
        self.assertEqual(resultinterval[1], datetime.date(year=2003, month=3, day=31))

        resultinterval = parse_interval('2000-02-28/P1Y', relative=True)
        self.assertEqual(resultinterval[0], datetime.date(year=2000, month=2, day=28))
        self.assertEqual(resultinterval[1], datetime.date(year=2001, month=2, day=28))

        resultinterval = parse_interval('1999-02-28/P1Y', relative=True)
        self.assertEqual(resultinterval[0], datetime.date(year=1999, month=2, day=28))
        self.assertEqual(resultinterval[1], datetime.date(year=2000, month=2, day=28))

        resultinterval = parse_interval('1999-03-01/P1Y', relative=True)
        self.assertEqual(resultinterval[0], datetime.date(year=1999, month=3, day=1))
        self.assertEqual(resultinterval[1], datetime.date(year=2000, month=3, day=1))

        resultinterval = parse_interval('P1Y/2001-02-28', relative=True)
        self.assertEqual(resultinterval[0], datetime.date(year=2001, month=2, day=28))
        self.assertEqual(resultinterval[1], datetime.date(year=2000, month=2, day=28))

        resultinterval = parse_interval('P1Y/2001-03-01', relative=True)
        self.assertEqual(resultinterval[0], datetime.date(year=2001, month=3, day=1))
        self.assertEqual(resultinterval[1], datetime.date(year=2000, month=3, day=1))

        resultinterval = parse_interval('1980-03-05T01:01:00/1981-04-05T01:01:00', relative=True)
        self.assertEqual(resultinterval[0], datetime.datetime(year=1980, month=3, day=5, hour=1, minute=1))
        self.assertEqual(resultinterval[1], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))

        resultinterval = parse_interval('1980-03-05T01:01:00/1981-04-05', relative=True)
        self.assertEqual(resultinterval[0], datetime.datetime(year=1980, month=3, day=5, hour=1, minute=1))
        self.assertEqual(resultinterval[1], datetime.date(year=1981, month=4, day=5))

        resultinterval = parse_interval('1980-03-05/1981-04-05T01:01:00', relative=True)
        self.assertEqual(resultinterval[0], datetime.date(year=1980, month=3, day=5))
        self.assertEqual(resultinterval[1], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))

        resultinterval = parse_interval('1980-03-05/1981-04-05', relative=True)
        self.assertEqual(resultinterval[0], datetime.date(year=1980, month=3, day=5))
        self.assertEqual(resultinterval[1], datetime.date(year=1981, month=4, day=5))

        resultinterval = parse_interval('1981-04-05/1980-03-05', relative=True)
        self.assertEqual(resultinterval[0], datetime.date(year=1981, month=4, day=5))
        self.assertEqual(resultinterval[1], datetime.date(year=1980, month=3, day=5))

        resultinterval = parse_interval('1980-03-05T01:01:00--1981-04-05T01:01:00', intervaldelimiter='--', relative=True)
        self.assertEqual(resultinterval[0], datetime.datetime(year=1980, month=3, day=5, hour=1, minute=1))
        self.assertEqual(resultinterval[1], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))

        resultinterval = parse_interval('1980-03-05 01:01:00/1981-04-05 01:01:00', datetimedelimiter=' ', relative=True)
        self.assertEqual(resultinterval[0], datetime.datetime(year=1980, month=3, day=5, hour=1, minute=1))
        self.assertEqual(resultinterval[1], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))

    def test_parse_interval_relative_suffixgarbage(self):
        #Don't allow garbage after the duration
        #https://bitbucket.org/nielsenb/aniso8601/issues/9/durations-with-trailing-garbage-are-parsed
        with self.assertRaises(ValueError):
            parse_interval('2001/P1Dasdf', relative=True)

        with self.assertRaises(ValueError):
            parse_interval('P1Dasdf/2001', relative=True)

        with self.assertRaises(ValueError):
            parse_interval('2001/P0003-06-04T12:30:05.5asdfasdf', relative=True)

        with self.assertRaises(ValueError):
            parse_interval('P0003-06-04T12:30:05.5asdfasdf/2001', relative=True)

class TestRepeatingIntervalParserFunctions(unittest.TestCase):
    def test_parse_repeating_interval(self):
        results = list(parse_repeating_interval('R3/1981-04-05/P1D'))
        self.assertEqual(results[0], datetime.date(year=1981, month=4, day=5))
        self.assertEqual(results[1], datetime.date(year=1981, month=4, day=6))
        self.assertEqual(results[2], datetime.date(year=1981, month=4, day=7))

        results = list(parse_repeating_interval('R11/PT1H2M/1980-03-05T01:01:00'))

        for dateindex in compat.range(0, 11):
             self.assertEqual(results[dateindex], datetime.datetime(year=1980, month=3, day=5, hour=1, minute=1) - dateindex * datetime.timedelta(hours=1, minutes=2))

        results = list(parse_repeating_interval('R2--1980-03-05T01:01:00--1981-04-05T01:01:00', intervaldelimiter='--'))
        self.assertEqual(results[0], datetime.datetime(year=1980, month=3, day=5, hour=1, minute=1))
        self.assertEqual(results[1], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))

        results = list(parse_repeating_interval('R2/1980-03-05 01:01:00/1981-04-05 01:01:00', datetimedelimiter=' '))
        self.assertEqual(results[0], datetime.datetime(year=1980, month=3, day=5, hour=1, minute=1))
        self.assertEqual(results[1], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))

        resultgenerator = parse_repeating_interval('R/PT1H2M/1980-03-05T01:01:00')

        for dateindex in compat.range(0, 11):
             self.assertEqual(next(resultgenerator), datetime.datetime(year=1980, month=3, day=5, hour=1, minute=1) - dateindex * datetime.timedelta(hours=1, minutes=2))

    def test_parse_repeating_interval_suffixgarbage(self):
        #Don't allow garbage after the duration
        #https://bitbucket.org/nielsenb/aniso8601/issues/9/durations-with-trailing-garbage-are-parsed
        with self.assertRaises(ValueError):
            parse_interval('R3/1981-04-05/P1Dasdf')

        with self.assertRaises(ValueError):
            parse_interval('R3/1981-04-05/P0003-06-04T12:30:05.5asdfasdf')

    def test_parse_repeating_interval_relative(self):
        results = list(parse_repeating_interval('R3/1981-04-05/P1D', relative=True))
        self.assertEqual(results[0], datetime.date(year=1981, month=4, day=5))
        self.assertEqual(results[1], datetime.date(year=1981, month=4, day=6))
        self.assertEqual(results[2], datetime.date(year=1981, month=4, day=7))

        results = list(parse_repeating_interval('R11/PT1H2M/1980-03-05T01:01:00', relative=True))

        for dateindex in compat.range(0, 11):
             self.assertEqual(results[dateindex], datetime.datetime(year=1980, month=3, day=5, hour=1, minute=1) - dateindex * datetime.timedelta(hours=1, minutes=2))

        results = list(parse_repeating_interval('R2--1980-03-05T01:01:00--1981-04-05T01:01:00', intervaldelimiter='--', relative=True))
        self.assertEqual(results[0], datetime.datetime(year=1980, month=3, day=5, hour=1, minute=1))
        self.assertEqual(results[1], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))

        results = list(parse_repeating_interval('R2/1980-03-05 01:01:00/1981-04-05 01:01:00', datetimedelimiter=' ', relative=True))
        self.assertEqual(results[0], datetime.datetime(year=1980, month=3, day=5, hour=1, minute=1))
        self.assertEqual(results[1], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))

        #Make sure relative is correctly applied for months
        #https://bitbucket.org/nielsenb/aniso8601/issues/12/month-intervals-calculated-incorrectly-or
        results = list(parse_repeating_interval('R4/2017-04-30T00:00:00/P1M', relative=True))
        self.assertEqual(results[0], datetime.datetime(year=2017, month=4, day=30))
        self.assertEqual(results[1], datetime.datetime(year=2017, month=5, day=30))
        self.assertEqual(results[2], datetime.datetime(year=2017, month=6, day=30))
        self.assertEqual(results[3], datetime.datetime(year=2017, month=7, day=30))

        resultgenerator = parse_repeating_interval('R/PT1H2M/1980-03-05T01:01:00', relative=True)

        for dateindex in compat.range(0, 11):
             self.assertEqual(next(resultgenerator), datetime.datetime(year=1980, month=3, day=5, hour=1, minute=1) - dateindex * datetime.timedelta(hours=1, minutes=2))

    def test_parse_repeating_interval_relative_suffixgarbage(self):
        #Don't allow garbage after the duration
        #https://bitbucket.org/nielsenb/aniso8601/issues/9/durations-with-trailing-garbage-are-parsed
        with self.assertRaises(ValueError):
            parse_interval('R3/1981-04-05/P1Dasdf', relative=True)

        with self.assertRaises(ValueError):
            parse_interval('R3/1981-04-05/P0003-06-04T12:30:05.5asdfasdf')

    def test_parse_repeating_interval_relative_nodateutil(self):
        import sys
        import dateutil

        dateutil_import = dateutil

        sys.modules['dateutil'] = None

        with self.assertRaises(RuntimeError):
            parse_repeating_interval('R3/1981-04-05/P1D', relative=True)

        with self.assertRaises(RuntimeError):
            parse_repeating_interval('R4/2017-04-30T00:00:00/P1M', relative=True)

        #Reinstall dateutil
        sys.modules['dateutil'] = dateutil

class TestIntervalPartParserFunctions(unittest.TestCase):
    def test_parse_interval_parts(self):
        resultinterval = _parse_interval_parts('P1M/1981-04-05T01:01:00')
        self.assertEqual(resultinterval[0], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))
        self.assertEqual(resultinterval[1], datetime.datetime(year=1981, month=3, day=6, hour=1, minute=1))
        self.assertEqual(resultinterval[2], -datetime.timedelta(days=30))

        resultinterval = _parse_interval_parts('P1M/1981-04-05')
        self.assertEqual(resultinterval[0], datetime.date(year=1981, month=4, day=5))
        self.assertEqual(resultinterval[1], datetime.date(year=1981, month=3, day=6))
        self.assertEqual(resultinterval[2], -datetime.timedelta(days=30))

        resultinterval = _parse_interval_parts('PT1H/2014-11-12')
        self.assertEqual(resultinterval[0], datetime.date(year=2014, month=11, day=12))
        self.assertEqual(resultinterval[1], datetime.datetime(year=2014, month=11, day=11, hour=23))
        self.assertEqual(resultinterval[2], -datetime.timedelta(hours=1))

        resultinterval = _parse_interval_parts('PT4H54M6.5S/2014-11-12')
        self.assertEqual(resultinterval[0], datetime.date(year=2014, month=11, day=12))
        self.assertEqual(resultinterval[1], datetime.datetime(year=2014, month=11, day=11, hour=19, minute=5, second=53, microsecond=500000))
        self.assertEqual(resultinterval[2], -datetime.timedelta(hours=4, minutes=54, seconds=6.5))

        resultinterval = _parse_interval_parts('1981-04-05T01:01:00/P1M1DT1M')
        self.assertEqual(resultinterval[0], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))
        self.assertEqual(resultinterval[1], datetime.datetime(year=1981, month=5, day=6, hour=1, minute=2))
        self.assertEqual(resultinterval[2], datetime.timedelta(days=31, minutes=1))

        resultinterval = _parse_interval_parts('1981-04-05/P1M1D')
        self.assertEqual(resultinterval[0], datetime.date(year=1981, month=4, day=5))
        self.assertEqual(resultinterval[1], datetime.date(year=1981, month=5, day=6))
        self.assertEqual(resultinterval[2], datetime.timedelta(days=31))

        resultinterval = _parse_interval_parts('2014-11-12/PT1H')
        self.assertEqual(resultinterval[0], datetime.date(year=2014, month=11, day=12))
        self.assertEqual(resultinterval[1], datetime.datetime(year=2014, month=11, day=12, hour=1, minute=0))
        self.assertEqual(resultinterval[2], datetime.timedelta(hours=1))

        resultinterval = _parse_interval_parts('2014-11-12/PT4H54M6.5S')
        self.assertEqual(resultinterval[0], datetime.date(year=2014, month=11, day=12))
        self.assertEqual(resultinterval[1], datetime.datetime(year=2014, month=11, day=12, hour=4, minute=54, second=6, microsecond=500000))
        self.assertEqual(resultinterval[2], datetime.timedelta(hours=4, minutes=54, seconds=6.5))

        resultinterval = _parse_interval_parts('1980-03-05T01:01:00/1981-04-05T01:01:00')
        self.assertEqual(resultinterval[0], datetime.datetime(year=1980, month=3, day=5, hour=1, minute=1))
        self.assertEqual(resultinterval[1], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))
        self.assertEqual(resultinterval[2], datetime.timedelta(days=396)) #One year, one month (30 days)

        resultinterval = _parse_interval_parts('1980-03-05T01:01:00/1981-04-05')
        self.assertEqual(resultinterval[0], datetime.datetime(year=1980, month=3, day=5, hour=1, minute=1))
        self.assertEqual(resultinterval[1], datetime.date(year=1981, month=4, day=5))
        self.assertEqual(resultinterval[2], datetime.timedelta(days=395, hours=22, minutes=59))

        resultinterval = _parse_interval_parts('1980-03-05/1981-04-05T01:01:00')
        self.assertEqual(resultinterval[0], datetime.date(year=1980, month=3, day=5))
        self.assertEqual(resultinterval[1], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))
        self.assertEqual(resultinterval[2], datetime.timedelta(days=396, hours=1, minutes=1))

        resultinterval = _parse_interval_parts('1980-03-05/1981-04-05')
        self.assertEqual(resultinterval[0], datetime.date(year=1980, month=3, day=5))
        self.assertEqual(resultinterval[1], datetime.date(year=1981, month=4, day=5))
        self.assertEqual(resultinterval[2], datetime.timedelta(days=396))

        resultinterval = _parse_interval_parts('1981-04-05/1980-03-05')
        self.assertEqual(resultinterval[0], datetime.date(year=1981, month=4, day=5))
        self.assertEqual(resultinterval[1], datetime.date(year=1980, month=3, day=5))
        self.assertEqual(resultinterval[2], -datetime.timedelta(days=396))

        resultinterval = _parse_interval_parts('1980-03-05T01:01:00--1981-04-05T01:01:00', intervaldelimiter='--')
        self.assertEqual(resultinterval[0], datetime.datetime(year=1980, month=3, day=5, hour=1, minute=1))
        self.assertEqual(resultinterval[1], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))
        self.assertEqual(resultinterval[2], datetime.timedelta(days=396))

        resultinterval = _parse_interval_parts('1980-03-05 01:01:00/1981-04-05 01:01:00', datetimedelimiter=' ')
        self.assertEqual(resultinterval[0], datetime.datetime(year=1980, month=3, day=5, hour=1, minute=1))
        self.assertEqual(resultinterval[1], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))
        self.assertEqual(resultinterval[2], datetime.timedelta(days=396))

    def test_parse_interval_parts_relative(self):
        resultinterval = _parse_interval_parts('P1M/1981-04-05T01:01:00', relative=True)
        self.assertEqual(resultinterval[0], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))
        self.assertEqual(resultinterval[1], datetime.datetime(year=1981, month=3, day=5, hour=1, minute=1))
        self.assertEqual(resultinterval[2], -dateutil.relativedelta.relativedelta(months=1))

        resultinterval = _parse_interval_parts('P1M/1981-04-05', relative=True)
        self.assertEqual(resultinterval[0], datetime.date(year=1981, month=4, day=5))
        self.assertEqual(resultinterval[1], datetime.date(year=1981, month=3, day=5))
        self.assertEqual(resultinterval[2], -dateutil.relativedelta.relativedelta(months=1))

        resultinterval = _parse_interval_parts('PT1H/2014-11-12', relative=True)
        self.assertEqual(resultinterval[0], datetime.date(year=2014, month=11, day=12))
        self.assertEqual(resultinterval[1], datetime.datetime(year=2014, month=11, day=11, hour=23))
        self.assertEqual(resultinterval[2], -dateutil.relativedelta.relativedelta(hours=1))

        resultinterval = _parse_interval_parts('PT4H54M6.5S/2014-11-12', relative=True)
        self.assertEqual(resultinterval[0], datetime.date(year=2014, month=11, day=12))
        self.assertEqual(resultinterval[1], datetime.datetime(year=2014, month=11, day=11, hour=19, minute=5, second=53, microsecond=500000))
        self.assertEqual(resultinterval[2], -dateutil.relativedelta.relativedelta(hours=4, minutes=54, seconds=6.5))

        resultinterval = _parse_interval_parts('1981-04-05T01:01:00/P1M1DT1M', relative=True)
        self.assertEqual(resultinterval[0], datetime.datetime(year=1981, month=4, day=5, hour=1, minute=1))
        self.assertEqual(resultinterval[1], datetime.datetime(year=1981, month=5, day=6, hour=1, minute=2))
        self.assertEqual(resultinterval[2], dateutil.relativedelta.relativedelta(months=1, days=1, minutes=1))

        resultinterval = _parse_interval_parts('1981-04-05/P1M1D', relative=True)
        self.assertEqual(resultinterval[0], datetime.date(year=1981, month=4, day=5))
        self.assertEqual(resultinterval[1], datetime.date(year=1981, month=5, day=6))
        self.assertEqual(resultinterval[2], dateutil.relativedelta.relativedelta(months=1, days=1))

        resultinterval = _parse_interval_parts('2014-11-12/PT1H', relative=True)
        self.assertEqual(resultinterval[0], datetime.date(year=2014, month=11, day=12))
        self.assertEqual(resultinterval[1], datetime.datetime(year=2014, month=11, day=12, hour=1, minute=0))
        self.assertEqual(resultinterval[2], dateutil.relativedelta.relativedelta(hours=1))

        resultinterval = _parse_interval_parts('2014-11-12/PT4H54M6.5S', relative=True)
        self.assertEqual(resultinterval[0], datetime.date(year=2014, month=11, day=12))
        self.assertEqual(resultinterval[1], datetime.datetime(year=2014, month=11, day=12, hour=4, minute=54, second=6, microsecond=500000))
        self.assertEqual(resultinterval[2], dateutil.relativedelta.relativedelta(hours=4, minutes=54, seconds=6.5))

        #Some relativedelta examples
        #http://dateutil.readthedocs.org/en/latest/examples.html#relativedelta-examples
        resultinterval = _parse_interval_parts('2003-01-27/P1M', relative=True)
        self.assertEqual(resultinterval[0], datetime.date(year=2003, month=1, day=27))
        self.assertEqual(resultinterval[1], datetime.date(year=2003, month=2, day=27))
        self.assertEqual(resultinterval[2], dateutil.relativedelta.relativedelta(months=1))

        resultinterval = _parse_interval_parts('2003-01-31/P1M', relative=True)
        self.assertEqual(resultinterval[0], datetime.date(year=2003, month=1, day=31))
        self.assertEqual(resultinterval[1], datetime.date(year=2003, month=2, day=28))
        self.assertEqual(resultinterval[2], dateutil.relativedelta.relativedelta(months=1))

        resultinterval = _parse_interval_parts('2003-01-31/P2M', relative=True)
        self.assertEqual(resultinterval[0], datetime.date(year=2003, month=1, day=31))
        self.assertEqual(resultinterval[1], datetime.date(year=2003, month=3, day=31))
        self.assertEqual(resultinterval[2], dateutil.relativedelta.relativedelta(months=2))

        resultinterval = _parse_interval_parts('2000-02-28/P1Y', relative=True)
        self.assertEqual(resultinterval[0], datetime.date(year=2000, month=2, day=28))
        self.assertEqual(resultinterval[1], datetime.date(year=2001, month=2, day=28))
        self.assertEqual(resultinterval[2], dateutil.relativedelta.relativedelta(years=1))

        resultinterval = _parse_interval_parts('1999-02-28/P1Y', relative=True)
        self.assertEqual(resultinterval[0], datetime.date(year=1999, month=2, day=28))
        self.assertEqual(resultinterval[1], datetime.date(year=2000, month=2, day=28))
        self.assertEqual(resultinterval[2], dateutil.relativedelta.relativedelta(years=1))

        resultinterval = _parse_interval_parts('1999-03-01/P1Y', relative=True)
        self.assertEqual(resultinterval[0], datetime.date(year=1999, month=3, day=1))
        self.assertEqual(resultinterval[1], datetime.date(year=2000, month=3, day=1))
        self.assertEqual(resultinterval[2], dateutil.relativedelta.relativedelta(years=1))

        resultinterval = _parse_interval_parts('P1Y/2001-02-28', relative=True)
        self.assertEqual(resultinterval[0], datetime.date(year=2001, month=2, day=28))
        self.assertEqual(resultinterval[1], datetime.date(year=2000, month=2, day=28))
        self.assertEqual(resultinterval[2], -dateutil.relativedelta.relativedelta(years=1))

        resultinterval = _parse_interval_parts('P1Y/2001-03-01', relative=True)
        self.assertEqual(resultinterval[0], datetime.date(year=2001, month=3, day=1))
        self.assertEqual(resultinterval[1], datetime.date(year=2000, month=3, day=1))
        self.assertEqual(resultinterval[2], -dateutil.relativedelta.relativedelta(years=1))

    def test_parse_interval_parts_relative_nodateutil(self):
        import sys
        import dateutil

        dateutil_import = dateutil

        sys.modules['dateutil'] = None

        with self.assertRaises(RuntimeError):
            _parse_interval_parts('P1M/1981-04-05T01:01:00', relative=True)

        #Reinstall dateutil
        sys.modules['dateutil'] = dateutil



try:
    from setuptools import setup
except ImportError:
    from distutils import setup

readme = open('README.rst', 'r')
README_TEXT = readme.read()
readme.close()

setup(
    name='aniso8601',
    version='2.0.0',
    description='A library for parsing ISO 8601 strings.',
    long_description=README_TEXT,
    author='Brandon Nielsen',
    author_email='nielsenb@jetfuse.net',
    url='https://bitbucket.org/nielsenb/aniso8601',
    extras_require={'relative': ['python-dateutil']},
    packages=['aniso8601'],
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)
